from sklearn.model_selection import train_test_split

class Model():
    def __init__(self, X, y, medias, organic, settings, model):

        self.settings = settings
        self.model = model
        self.media_labels = medias
        self.media_labels = organic

        X_train, X_test, y_train, y_test = train_test_split(X, y)

        self.X_actual = X
        self.y_actual = y
        self.X_train = X_train
        self.y_train = y_train
        self.X_test = X_test
        self.y_test = y_test

    def fit(self, y, X):
        return None

    # def results(self):
    #     results_df = pd.DataFrame(self.contribution().sum(), columns=['contribution'])
    #     results_df['share'] = results_df['contribution'] / results_df['contribution'].sum() * 100
    #     results_df['coefficient'] = self.coefficients
    #     results_df['pvalue'] = self._pvalues()
    #     results_df = pd.concat([results_df, self._confidence_intervals()], axis=1)
    #
    #     return np.around(results_df, 3)
    #
    # def contribution(self, X=None):
    #     if (X) is None:
    #         X = self.X_actual
    #
    #     coef_df = pd.DataFrame({'coefficient': self.coefficients}, index=X.columns)
    #
    #     data = []
    #     for x in list(X.columns):
    #         contrib = coef_df['coefficient'].loc[x] * X[x]
    #         data.append(contrib)
    #
    #     contrib_df = pd.DataFrame(data).T
    #
    #     return contrib_df
    #
    # def predict(self, X=None):
    #     contribution = self.contribution(X)
    #     y_pred = contribution.sum(axis=1)
    #     return y_pred
    #
    # def metrics(self, metric_labels):
    #     metrics = []
    #     for metric in metric_labels:
    #         value = check_metric(metric, self)
    #         metrics.append((metric, value))
    #     for label, output in metrics:
    #         print(f"{output[1]} {label}: {output[0]}")
    #
    # def metric(self, metric_label):
    #     value = check_metric(metric_label, self)
    #     return value[0]

    # def _save(self):
    #     file = self.settings['file']
    #     file_paths = file.split('/')
    #     filename = file_paths.pop()
    #     file_paths.append("models-" + filename)
    #     file_loc = '/'.join(file_paths)
    #     models_output = pd.DataFrame.from_dict([{
    #         'file': self.settings['file'],
    #         'model': self.model,
    #         'metric': self.settings['metric'],
    #         'error': self.metric(self.settings['metric']),
    #         'timestamp': dt.datetime.today().strftime('%Y-%m-%d %H:%M'),
    #         'runtime': self.runtime,
    #         'y_label': self.y_train.name,
    #         'X_labels': ', '.join(list(self.X_train.columns)),
    #     }])
    #     if os.path.isfile(file_loc):
    #         # save to existing file
    #         loaded_models = pd.read_csv(file_loc)
    #         all_models = loaded_models.append(models_output)
    #         all_models.to_csv(file_loc, index=False)
    #         print("added model to existing file")
    #     else:
    #         # save new model file
    #         models_output.to_csv(file_loc, index=False)
    #         print("added new model file locally")

    # def show(self, charts=True, metrics=True, results=True):
    #     accuracy(self.y_actual, self.predict()) if charts else False
    #     self.metrics(["rsquared", "nrmse", "mape", "decomp-rssd", "cond-no"]) if metrics else False
    #     display(self.results()) if results else False











from src.cassandra.data import show_prediction_vs_actual_graph_with_error
from src.cassandra.helper import helper_nevergrad
from model.deepLearning import deepLearning
from model.linear import linear
from model.logLinear import logLinear
from src.cassandra.model.modelEvaluation.evaluation import show_coefficients
from model.ridge import ridge


def selection_model(df, medias, organic, X, y, settings_models):
    # funziona
    if 'linear' in settings_models['model']:
        result_linear, model_linear = linear(df, medias, organic, X, y, 'Linear', settings_models['linear_metric'])
        show_prediction_vs_actual_graph_with_error(result_linear, y, result_linear['prediction'], 'ordine_data', 'revenue', 'Linear')
        if settings_models['linear_coeffs'] == True:
            show_coefficients(X.columns, model_linear, 'Linear', graph=True)

    # chiedere feadback se funziona
    elif 'logLinear' in settings_models['model']:
        result_log_linear, model_log_linear = logLinear(df, medias, organic, X, y, 'Log Linear', settings_models['log_linear_metric'])
        show_prediction_vs_actual_graph_with_error(result_log_linear, np.log(y), result_log_linear['prediction'], 'ordine_data',
                                         'revenue', 'Log Linear')
        if settings_models['log_linear_coeffs'] == True:
            show_coefficients(X.columns, model_log_linear, 'Log Linear', graph=True)

    # funziona
    elif 'ridge' in settings_models['model']:
        result_ridge, model_ridge = ridge(df, medias, organic, X, y, 'Ridge', settings_models['ridge_metric'])
        show_prediction_vs_actual_graph_with_error(result_ridge, y, result_ridge['prediction'], 'ordine_data', 'revenue', 'Ridge',)
        if settings_models['ridge_coeffs'] == True:
            show_coefficients(X.columns, model_ridge, 'Ridge',graph=True)

    # da capire e chiedere feadback se funziona
    elif 'deepLearning' in settings_models['model']:
        result_deep_learning, model_deep_learning = deepLearning(df, medias, organic, X, y, 'Deep Learning',
                                                                 settings_models['deep_learning_metric'])
        show_prediction_vs_actual_graph_with_error(result_deep_learning, y, result_deep_learning['prediction'], 'ordine_data',
                                         'revenue', 'Deep Learning')
        if settings_models['deep_learning_coeffs']:
            show_coefficients(X.columns, model_deep_learning, 'Deep Learning', graph=True)

    elif 'nevergrad' in settings_models['model']:
        helper_nevergrad()

    elif 'all' in settings_models['model']:
    #LINEAR
        result_linear, model_linear = linear(df, medias, organic, X, y, 'Linear', settings_models['linear_metric'])
        show_prediction_vs_actual_graph_with_error(result_linear, y, result_linear['prediction'], 'ordine_data', 'revenue',
                                         'Linear')
        if settings_models['linear_coeffs'] == True:
            show_coefficients(X.columns, model_linear, 'Linear', graph=True)

    #LOG LINEAR
        result_log_linear, model_log_linear = logLinear(df, medias, organic, X, y, 'Log Linear',
                                                        settings_models['log_linear_metric'])
        show_prediction_vs_actual_graph_with_error(result_log_linear, np.log(y), result_log_linear['prediction'],
                                         'ordine_data',
                                         'revenue', 'Log Linear')
        if settings_models['log_linear_coeffs'] == True:
            show_coefficients(X.columns, model_log_linear, 'Log Linear', graph=True)
    #RIDGE
        result_ridge, model_ridge = ridge(df, medias, organic, X, y, 'Ridge', settings_models['ridge_metric'])
        show_prediction_vs_actual_graph_with_error(result_ridge, y, result_ridge['prediction'], 'ordine_data', 'revenue',
                                         'Ridge', )
        if settings_models['ridge_coeffs'] == True:
            show_coefficients(X.columns, model_ridge, 'Ridge', graph=True)

    #DEEP LEARNING
        result_deep_learning, model_deep_learning = deepLearning(df, medias, organic, X, y, 'Deep Learning',
                                                                 settings_models['deep_learning_metric'])
        show_prediction_vs_actual_graph_with_error(result_deep_learning, y, result_deep_learning['prediction'], 'ordine_data',
                                         'revenue', 'Deep Learning')
        if settings_models['deep_learning_coeffs']:
            show_coefficients(X.columns, model_deep_learning, 'Deep Learning', graph=True)

    #NEVERGRAD
        helper_nevergrad()



from sklearn.linear_model import LinearRegression
from model.modelClasses import Model
from src.cassandra.data import create_model


class Linear(Model):
    def __init__(self, X, y, medias, organic, settings):
        # inheritance and start timer
        super().__init__(X, y, medias, organic, settings, "Linear")

        # fit the model
        self.model = self.fit()



    # fit the model
    def fit(self):
        return create_model(self.medias, self.organic, LinearRegression()).fit(self.X_train, self.y_train)



# def test_read_csv():
#     csv_data = pd.read_csv('D:/workspace_cassandra/mmm/spedire/dataset-spedire-07-01-22.csv')
#     csv_data = csv_data[['ordine_data', 'revenue', 'transazioni', 'google_search_spent', 'google_performance_max_spent',
#                          'google_display_spent', 'fb_retargeting_spent', 'fb_prospecting_spent', 'bing_spent',
#                          'bing_organico', 'google_organico', 'referral', 'email_sessioni', 'cambio_eur_dollaro',
#                          'ios_14', 'Offerte']]
#
#     assert read_csv('D:/workspace_cassandra/mmm/spedire/dataset-spedire-07-01-22.csv',',',['ordine_data','revenue','transazioni','google_search_spent','google_performance_max_spent','google_display_spent','fb_retargeting_spent','fb_prospecting_spent','bing_spent','bing_organico','google_organico','referral','email_sessioni','cambio_eur_dollaro','ios_14','Offerte'],[]) == csv_data
